/**
 * Author : Dmitry Abramov aka A6PAMOB
 * email  : A6PAMOB@yandex.ru
 * 
 */
package decompile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

public class Unscript {

	public static int DEBUG = 0;

	public static int INT_SIZE = 4;
	public static int WORD_SIZE = 2;
	public static int START_MAIN = 4 * INT_SIZE;
	public static int MAIN_NAME_SIZE = 32;
	public static int MAIN_SIZE = MAIN_NAME_SIZE + INT_SIZE + INT_SIZE;
	public static int SUB_NAME_SIZE = 32;
	public static int SUB_SIZE = SUB_NAME_SIZE + INT_SIZE + INT_SIZE;

	final static int SCIRPT_PARSER_VALUE_TYPE_INVALID = 0;
	final static int SCIRPT_PARSER_VALUE_TYPE_SINGLE_WORD = 1;
	final static int SCIRPT_PARSER_VALUE_TYPE_STRING = 2;
	final static int SCIRPT_PARSER_VALUE_TYPE_MULTI_WORD = 3;
	final static int SCIRPT_PARSER_VALUE_TYPE_GPIO_WORD = 4;
	final static int SCIRPT_PARSER_VALUE_TYPE_NULL = 5;

	public static byte buffer[];

	private static int readInt(int pos) {

		byte b1 = buffer[pos];
		byte b2 = buffer[pos + 1];
		byte b3 = buffer[pos + 2];
		byte b4 = buffer[pos + 3];

		return (b4 << 24) + ((b3 & 0xFF) << 16) + ((b2 & 0xFF) << 8)
				+ (b1 & 0xFF);
	}

	private static int readWord(int pos) {

		byte b1 = buffer[pos];
		byte b2 = buffer[pos + 1];

		return ((b2 & 0xFF) << 8) + (b1 & 0xFF);
	}

	private static String readString(int pos, int size) {
		byte[] b = new byte[size];

		for (int i = 0; i < size; i++)
			b[i] = buffer[pos + i];

		return new String(b);
	}

	public static void main(String[] args) {

		// Check params
		if (args.length == 0) {
			System.out.println("Usage: unscript <path.to.bin.file>");
			return;
		}

		// Read a bin file
		File f = new File(args[0]);
		int fSize = (int) f.length();
		if (fSize == 0) {
			System.out.println("The bin file is too small :-)");
			return;
		}

		buffer = new byte[fSize];
		FileInputStream fi = null;
		try {
			fi = new FileInputStream(f);
			fi.read(buffer);
			fi.close();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		} catch (IOException e) {
			e.printStackTrace();
			if (fi != null)
				try {
					fi.close();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			return;
		}

		// Parse
		int countMain = readInt(0);

		for (int main = 0; main < countMain; main++) {
			String name = readString(START_MAIN + main * MAIN_SIZE,
					MAIN_NAME_SIZE);
			int countSub = readInt(START_MAIN + main * MAIN_SIZE
					+ MAIN_NAME_SIZE);
			int offset = readInt(START_MAIN + main * MAIN_SIZE + MAIN_NAME_SIZE
					+ INT_SIZE);
			offset = offset * 4;

			if (DEBUG == 0)
				System.out.println("[" + name.trim() + "]");
			else
				System.out.println("[" + name.trim() + "] ; offset:"
						+ Integer.toHexString(offset));

			for (int sub = 0; sub < countSub; sub++) {
				String nameSub = readString(offset + sub * SUB_SIZE,
						SUB_NAME_SIZE);
				int offsetSub = readInt(offset + sub * SUB_SIZE + SUB_NAME_SIZE);
				offsetSub = offsetSub * 4;
				/*
				int aTwoValue = readInt(offset + sub * SUB_SIZE + SUB_NAME_SIZE
						+ INT_SIZE);
				int pattern = (aTwoValue >> 16) & 0xFFFF;
				int wordCount = (aTwoValue) & 0xFFFF;
				*/
				int wordCount= readWord(offset + sub * SUB_SIZE + SUB_NAME_SIZE
						+ INT_SIZE);
				int pattern = readWord(offset + sub * SUB_SIZE + SUB_NAME_SIZE
						+ INT_SIZE+WORD_SIZE);
				String aValue = "";
				switch (pattern) {
				case SCIRPT_PARSER_VALUE_TYPE_SINGLE_WORD:
					int iValue = readInt(offsetSub);

					if ((iValue > 0x0000FFFF) || (nameSub.contains("_addr")))
						aValue = "0x" + Integer.toHexString(iValue);
					else
						aValue = String.valueOf(iValue);

					break;

				case SCIRPT_PARSER_VALUE_TYPE_STRING:
					aValue = "\"" + readString(offsetSub, wordCount * 4).trim()
							+ "\"";
					break;

				case SCIRPT_PARSER_VALUE_TYPE_GPIO_WORD:
					char v1;
					String v2,
					v3,
					v4,
					v5,
					v6;
					int val;
					v1 = (char) (readInt(offsetSub + INT_SIZE * 0) + 64);
					v2 = String.valueOf(readInt(offsetSub + INT_SIZE * 1));

					val = readInt(offsetSub + INT_SIZE * 2);
					if (val == -1)
						v3 = "default";
					else
						v3 = String.valueOf(val);

					val = readInt(offsetSub + INT_SIZE * 3);
					if (val == -1)
						v4 = "default";
					else
						v4 = String.valueOf(val);

					val = readInt(offsetSub + INT_SIZE * 4);
					if (val == -1)
						v5 = "default";
					else
						v5 = String.valueOf(val);

					val = readInt(offsetSub + INT_SIZE * 5);
					if (val == -1)
						v6 = "default";
					else
						v6 = String.valueOf(val);

					aValue = "port:P" + v1 + "" + v2 + "<" + v3 + "><" + v4
							+ "><" + v5 + "><" + v6 + ">";
					break;
				default:
				}

				if (DEBUG == 0)
					System.out.println(nameSub.trim() + " = " + aValue.trim());
				else
					System.out.println(nameSub.trim() + " = " + aValue.trim()
							+ " ; offset=" + Integer.toHexString(offsetSub)
							+ "; pattern=" + pattern + "; wordcount="
							+ wordCount);

			}

			System.out.println();
		}

	}
}
